#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import time

import aiohttp
from aiohttp import web

from fast_tracker import agent
from fast_tracker import config


async def handle(request):
    name = request.match_info.get('name', "Anonymous")

    async with aiohttp.ClientSession() as session:
        async with session.get('http://user:pass@provider:9091/{name}') as response:
            time.sleep(.5)
            json = await response.json()
            return web.Response(text=str(json))


app = web.Application()
app.add_routes([web.get('/', handle), web.get('/{name}', handle)])

if __name__ == '__main__':
    config.service_name = 'consumer'
    config.logging_level = 'DEBUG'
    agent.start()

    web.run_app(app, port=9090)
